/*
 * @OSF_COPYRIGHT@
 * COPYRIGHT NOTICE
 * Copyright (c) 1990, 1991, 1992, 1993, 1996 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (DCE).  See the file named COPYRIGHT.DCE in the
 * src directory for the full copyright text.
 */
/*
 * HISTORY
 * $Log: iattrc.c,v $
 * Revision 1.1.4.2  1996/02/17  23:20:35  marty
 * 	Update OSF copyright year
 * 	[1996/02/17  22:39:19  marty]
 *
 * Revision 1.1.4.1  1995/12/11  20:04:59  root
 * 	Submit OSF/DCE 1.2.1
 * 	[1995/12/11  19:43:18  root]
 * 
 * Revision 1.1.2.3  1993/01/25  18:04:30  bbelch
 * 	Fix duplicate log
 * 	[1993/01/25  17:36:10  bbelch]
 * 
 * Revision 1.1.2.2  1993/01/11  22:14:13  bbelch
 * 	Embedding copyright notice
 * 	[1993/01/06  18:06:14  bbelch]
 * 
 * Revision 1.1  1992/01/19  03:35:08  devrcs
 * 	Initial revision
 * 
 * $EndLog$
 */
/*
#
#
# Copyright (c) 1991 by
#    Hewlett-Packard Company, Palo Alto, Ca.
#
# Permission to use, copy, modify, and distribute this
# software and its documentation for any purpose and without
# fee is hereby granted, provided that the above copyright
# notice appear in all copies and that both that copyright
# notice and this permission notice appear in supporting
# documentation, and that the name of Hewlett-Packard Company not
# be used in advertising or publicity pertaining to distribution
# of the software without specific, written prior permission.
# Hewlett-Packard Company makes no representations about the
# suitability of this software for any purpose.  It is provided
# "as is" without express or implied warranty.
#
#
# DCE API Test
#
*/



#include <iattr.h>
#include <iattrc.h>
#include <tsh_common.h>
#include <stdio.h>
#include <math.h>
#include <dce/rpc.h> 
#include <dce/stubbase.h> 

#define ARGS_REQUIRED 3

ndr_long_int TestFunction(ctx, argc, argv)
CtxBlock *ctx;
ndr_long_int argc;
unsigned_char_p_t argv[];
{
    ndr_long_int subcase;
    unsigned_char_p_t bound_string;
    error_status_t status;
    handle_t h;


    /**************************************************************************/
    /* Check the number of arguments passed in.  They should be exactly the   */
    /* required number when you come into this routine. Looking for a subcase,*/
    /* a bogus expected_status, and a hostname.                               */
    /**************************************************************************/
    if(argc != ARGS_REQUIRED)
    {
        TSHMessage(ctx, MOFAILURE, "Expecting %d arguments, got %d\n", ARGS_REQUIRED, argc);
        return(TSH_BAD_ARG_COUNT);
    }

    subcase = atol(argv[0]);         /* turn subcase string into an int.      */
                                     /* ignore argv[2] expected status value, */
                                     /* each test does its own error checking.*/
    return(interface_attributes_test(ctx, subcase));
}

int interface_attributes_test (ctx, subcase)
CtxBlock *ctx;
ndr_long_int subcase;
{
    int af;
    error_status_t st;
    uuid_t u;
    rpc_if_rep_t *for_real;
    rpc_endpoint_vector_elt_t *endpoint_pointer;
    unsigned char *string;

    for_real = (rpc_if_rep_t *)iattrtest_v1234_0_c_ifspec;
    endpoint_pointer = for_real->endpoint_vector.endpoint_vector_elt;
    switch(subcase)
    {

        case IATTR_GENERIC_TESTS:
        /**********************************************************************/
        /* see if the generic information generated by nidl is correct as it  */
        /* is stored in the if_ structure.                                    */
        /**********************************************************************/
        if (for_real->vers != 1234) 
        {
            TSHStatus(ctx, TEST,"incorrect version",FAILURE,SUCCESS);
            TSHMessage (ctx, MOWARNING, "for_real->vers != %d\n",
                                                 for_real->vers);
            return (TSH_INVALID_ARGUMENT);
        }
        if (for_real->opcnt != 3) 
        {
            TSHStatus(ctx, TEST,"incorrect opcnt",FAILURE,SUCCESS);
            TSHMessage (ctx, MOWARNING, "for_real->opcnt != %d\n",
                                                 for_real->opcnt);
            return (TSH_INVALID_ARGUMENT);
        }

        uuid_from_string((unsigned_char_t *)"41248b49-d000-0000-0d00-00c366000000", &u, &st);
        if (st != uuid_s_ok)
        {
            TSHStatus(ctx, TEST,"uuid_from_string failed",FAILURE,SUCCESS);
            return(st);
        }
        else
        {
            if (!uuid_equal (&for_real->id, &u, &st)) 
            {
                if (st != uuid_s_ok)
                {
                    TSHStatus(ctx, TEST,"uuid_equal failed",FAILURE,SUCCESS);
                    return(st);
                }
                TSHStatus(ctx, TEST,"incorrect uuid stored",FAILURE,SUCCESS);
                TSHMessage (ctx, MOWARNING, 
                "ifspec.id != 41248b49-d000-0000-0d00-00c366000000\n");
                return (TSH_INVALID_ARGUMENT);
            }
        }
        return(SUCCESS);
        break;
    
        case IATTR_NCACN_IP_TCP:
        string = (unsigned_char_t *)"6000";
        break;
    
        case IATTR_NCACN_DNET_NSP:
        string = (unsigned_char_t *)"6001";
        endpoint_pointer = (rpc_endpoint_vector_elt_t *)
                ((int)endpoint_pointer + sizeof(rpc_endpoint_vector_elt_t));
        break;

        case IATTR_NCACN_OSI_NSP:
        string = (unsigned_char_t *)"6002";
        endpoint_pointer = (rpc_endpoint_vector_elt_t *)
                ((int)endpoint_pointer + 2 * sizeof(rpc_endpoint_vector_elt_t));
        break;

        case IATTR_NCACN_OSI_TP4:
        string = (unsigned_char_t *)"6003";
        endpoint_pointer = (rpc_endpoint_vector_elt_t *)
                ((int)endpoint_pointer + 3 * sizeof(rpc_endpoint_vector_elt_t));
        break;

        case IATTR_NCADG_IP_UDP:
        string = (unsigned_char_t *)"6004";
        endpoint_pointer = (rpc_endpoint_vector_elt_t *)
                ((int)endpoint_pointer + 4 * sizeof(rpc_endpoint_vector_elt_t));
        break;

        case IATTR_NCADG_DDS:
        string = (unsigned_char_t *)"6005";
        endpoint_pointer = (rpc_endpoint_vector_elt_t *)
                ((int)endpoint_pointer + 5 * sizeof(rpc_endpoint_vector_elt_t));
        break;

        case IATTR_NCATP_IP_TCP:
        string = (unsigned_char_t *)"6006";
        endpoint_pointer = (rpc_endpoint_vector_elt_t *)
                ((int)endpoint_pointer + 6 * sizeof(rpc_endpoint_vector_elt_t));
        break;

        default:
        TSHStatus(ctx, TEST,"invalid sub case",FAILURE,SUCCESS);
        return(TSH_UNKNOWN_SUBCASE);

    }
    /**********************************************************************/
    /* see if the ports information generated by nidl is correct as it    */
    /* is stored in the if_ structure.                                    */
    /**********************************************************************/
    TSHMessage (ctx, MOTRACE, 
       "endpoint %s ? %s\n",endpoint_pointer->endpoint, string);
    if (strcmp (endpoint_pointer->endpoint, string))
    {
        TSHStatus(ctx, TEST,"endpoint is wrong",FAILURE,SUCCESS);
        TSHMessage (ctx, MOWARNING,
            "endpoint_pointer->endpoint %s != %s\n",
                                         endpoint_pointer->endpoint, string);
        return(TSH_INVALID_ARGUMENT);
    }
    return(SUCCESS);
}
